# -*- coding: utf-8 -*-
"""
Created on Wed Sep 20 10:22:56 2023

@author: Ross James

Script for calling in to submit jobs.
The script will creat a directory from the base name of the job,
move the associated files to that directory and then submit them.

Usage:
    
from job_submitter import JobSubmitter
    
job_submitter = JobSubmitter()
job_submitter.submit_job(["path/to/your/task.sh"], additional_ext: list else None)

╭╮╱╭┳━━━┳━━━╮╱╱╭━━━╮╱╱╭╮╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╭━━━━╮╱╱╱╱╭╮
┃┃╱┃┃╭━╮┃╭━╮┃╱╱┃╭━╮┃╱╱┃┃╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱╱┃╭╮╭╮┃╱╱╱╱┃┃
┃╰━╯┃╰━╯┃┃╱╰╯╱╱┃╰━━┳╮╭┫╰━┳╮╭┳┳━━┳━━┳┳━━┳━╮╱╱╰╯┃┃┣┻━┳━━┫┃╭━━╮
┃╭━╮┃╭━━┫┃╱╭┳━━╋━━╮┃┃┃┃╭╮┃╰╯┣┫━━┫━━╋┫╭╮┃╭╮┳━━╮┃┃┃╭╮┃╭╮┃┃┃━━┫
┃┃╱┃┃┃╱╱┃╰━╯┣━━┫╰━╯┃╰╯┃╰╯┃┃┃┃┣━━┣━━┃┃╰╯┃┃┃┣━━╯┃┃┃╰╯┃╰╯┃╰╋━━┃
╰╯╱╰┻╯╱╱╰━━━╯╱╱╰━━━┻━━┻━━┻┻┻┻┻━━┻━━┻┻━━┻╯╰╯╱╱╱╰╯╰━━┻━━┻━┻━━╯
"""

import os
import pathlib
import shutil

class JobSubmitter:
    def __init__(self):
        self.submitted = 0
        self.i = 0
        self.extensions = [".inp", 
                           ".sh", 
                           ".out", 
                           ".opt", 
                           "_original.opt", 
                           "_original.out", 
                           ".xyz"]
        
    def dos2unix(self, file):
        """
        Function to convert dos (Windows) created files to unix based files.
        
        Parameters
        ----------
        file : str
            Path to file.

        Returns
        -------
        unix file or None.

        """
        with open(file, 'rb') as f:
            content = f.read()
            if b'\r\n' in content:
                os.system(f'dos2unix {file}')
            f.close()

    def submit_jobs(self, tasks: list, additional_ext = None):
        
        """
        Function to convert dos (Windows) created files to unix based files.
        
        Parameters
        ----------
        tasks : list
            List of paths to sbatch files.
        aidditional_ext: list
            List of additional file extensions to look for
            default: None

        Returns
        -------
        Submits jobs, returns nothing

        """

        if additional_ext is not None: # add additional extensions to search pool
            self.extensions += additional_ext
        
        for task in tasks: # loop over .sh files
            task_path = pathlib.Path(task) # get path
            name = task_path.stem.replace(".sh", "")
            stripped_name = name
            

            cdir = os.path.abspath(".") # set working directory
            folder = task_path.parent # get folder that the task is in

            os.chdir(folder) # move to that folder
            if os.name == 'posix': # check if the system is unix based
                self.dos2unix(task) # if so then run dos2unix on the file
            stripped_folder = os.path.join(folder, stripped_name) # get a name for the task folder
            os.makedirs(stripped_folder, exist_ok=True) # make the task folder

            for ext in self.extensions: # loop over extensions
                source = f"{name}{ext}"
                if os.path.exists(source): # if the file exists move the file to the task folder
                    shutil.move(source, os.path.join(stripped_folder, source))

            os.chdir(stripped_folder) # move to the task folder
            os.system(f"sbatch {name}.sh") # submit the job
            os.chdir(cdir) # move back to the working directory

            self.submitted += 1 # add 1
            self.i += 1 # add 1
            print(f"{self.i} Jobs have been submitted\n")


# -*- coding: utf-8 -*-
"""
Created on Wed Sep 20 11:02:09 2023

@author: bwb16179
"""

import os
import pathlib
import shutil
import ase.io

# import submodules

from HPC_submission_tools.JobSubmission import *
from HPC_submission_tools.XYZ2ORCA import *


# -*- coding: utf-8 -*-
"""
Created on Wed Sep 20 11:04:42 2023

@author: bwb16179
"""

from distutils.core import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'HPC_submission_tools',         # How you named your package folder (MyLib)
  packages = ['HPC_submission_tools'],   # Chose the same as "name"
  version = '1.5',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Tools pertaining to use of HPC clusters with slurm file systems',   # Give a short description about your library
  long_description=long_description,
  long_description_content_type='text/markdown',
  author = 'Ross Urquhart',                   # Type in your name
  author_email = 'ross.urquhart@strath.ac.uk',      # Type in your E-Mail
  url = 'https://github.com/RossJamesUrquhart/HPC_submission_tools',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/RossJamesUrquhart/HPC_submission_tools/archive/refs/tags/v1.5.tar.gz',    # I explain this later on
  keywords = ['HPC', 'Command line'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',      #Specify which pyhton versions that you want to support
  ],
)


# -*- coding: utf-8 -*-
"""
Created on Wed May  4 13:57:18 2022

@author: bwb16179
"""
from ase.io import read

def buffer(string, L, end=True):
    while len(string) < L:
        string = string+"0"
    return string

class XYZ2ORCA:
    
    def __init__(self, XYZ: list, job: str, functional: str, basis_set: str, charge: int, solvent: str, cores: int, time: str, kwargs: list):
        self.XYZ = XYZ # path of xyz file (string) i.e. path/to/job
        self.job = job # Type of DFT job i.e. (string) Opt Freq, OptTS, etc.
        self.functional = functional # Functional for the job i.e. (string)  wB97X
        self.basis_set = basis_set # Basis set for the job, i.e. (string) Def2-SVP
        self.charge = charge # Charge of the system, i.e. (integer) 1
        self.solvent = solvent # solvent system i.e. (string) CPCM(Water)
        self.cores = cores # Number of cores for the job (integer), i.e. 4
        self.time = time # Time for the job i.e. (string) 02:00:00 (hours:minutes:seconds)
        self.add_kwargs = kwargs # additional keyword arguements i.e. list of strings - dispersion, SCF conv, etc.
        self.CPU = """#!/bin/bash
#SBATCH --export=ALL
#SBATCH --job-name={}
#SBATCH --account=tuttle-rmss
#SBATCH --partition=standard
#SBATCH --time="{}"
#SBATCH --ntasks={} --nodes=1
module purge
module load orca/5.0.4
    
/opt/software/scripts/job_prologue.sh\n
    """
        self.ElectronNos = {"H" : 1, "B" : 5, "C" : 6, "N" : 7, "O" : 8,
                            "F" : 9, "P" : 15, "S" : 16, "Cl" : 17, "Ir" : 77}
    
    def xyz2orca(self):
        frames = read(self.XYZ, index=":") # read in xyz file
        
        if not frames: # Sanity check, if empty then quit
            return False
        
        species = frames[0].get_chemical_symbols() # get elements in molecule
        
        CPUS = self.cores # number of CPU cores
        
        PAL = f"%PAL NPROCS {CPUS} END" if CPUS > 1 else "" # add in parallel line if needed
        
        strT = self.time # hours:minutes:seconds
        
        hours, mins, secs = map(int, strT.split(":")) # split time up
        
        if hours > 168: # check job won't sit in queue indefinitely
            print("Time is more than partition limit, must be <= 168 hours")
            print("Setting time to max available: 168 hours")
            strT = "168:00:00"
        
        for i,frame in enumerate(frames): # loop over the frames (we only want the first though)
            jobname = self.XYZ.replace(".xyz", ".inp") # Set a job name
            inp = jobname.split("/")[-1] # input file
            out = jobname.split("/")[-1].replace(".inp", ".out") # output file
            xyz = self.XYZ.split("/")[-1] # XYZ file
    
            with open(jobname.replace(".inp", ".sh"), 'w') as sbatch: # write an sbatch file
                sbatch.write(self.CPU.format(inp.replace(".inp", ""), strT, str(CPUS))) # write in the formatted Preamble for the job 

                charge = self.charge # set charge
                solvent = self.solvent # set solvent
                species = frame.get_chemical_symbols() # get species
                electrons = sum(self.ElectronNos.get(atom, 0) for atom in species) # get number of electrons in molecule
                multiplicity = str(2) if electrons % 2 == 1 else str(1)
                multiplicity = str(1) if (multiplicity == "2" and charge == 1) else str(2) # set multiplicity

                with open(jobname, 'w') as f: # open input file
                    functional = self.functional
                    basis_set = self.basis_set
                    job = self.job
                    add_kwargs = " ".join(self.add_kwargs)

                    f.write(f"""# ORCA 5.0.4 - {jobname} # write input file
# Basic Mode
#
! {job} {functional} {basis_set} {add_kwargs} {solvent}

{PAL}

%maxcore 4500

%geom
Maxiter 150
END

* xyzfile {str(charge)} {multiplicity} {xyz}
""")
                sbatch.write("/opt/software/orca/5.0.4/orca {} > {}\n".format(inp, out)) # write output file end
                sbatch.write("\n/opt/software/scripts/job_epilogue.sh")

        return True
        
    def FilterChemistry(self, atomlist): # check atoms in molecule are allowed under defined parameters
        species_order = ["H", "C", "N", "O"]
        return all(atom in species_order for atom in atomlist)